@@ -13,8 +13,7 @@
 #  limitations under the License.
 
 import enum
-from collections.abc import Mapping
-from typing import Sequence, Union
+from typing import Mapping, Sequence, Union
 
 
 class DateFormat(enum.Enum):
@@ -32,8 +31,11 @@ class DateFormat(enum.Enum):
 
 
 class DynamoDBType(enum.Enum):
-    NUMBER = enum.auto()
-    STRING = enum.auto()
+    NUMBER = 'N'
+    NUMBER_SET = 'NS'
+    STRING = 'S'
+    STRING_SET = 'SS'
+    BINARY_SET = 'BS'
 
 
 class PythonNumber(enum.Enum):
